package exnihilocreatio.tiles;

import com.google.common.base.Objects;
import exnihilocreatio.ModEnchantments;
import exnihilocreatio.blocks.BlockSieve;
import exnihilocreatio.config.ModConfig;
import exnihilocreatio.registries.manager.ExNihiloRegistryManager;
import exnihilocreatio.registries.types.Siftable;
import exnihilocreatio.util.BlockInfo;
import exnihilocreatio.util.Util;
import lombok.Getter;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.enchantment.EnchantmentHelper;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Enchantments;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.text.Style;
import net.minecraft.util.text.TextComponentString;
import net.minecraft.util.text.TextFormatting;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.UUID;

public class TileSieve extends BaseTileEntity {

    private static Random rand = new Random();
    public TileAutoSifter autoSifter = null;
    @Getter
    private BlockInfo currentStack;
    @Getter
    private byte progress = 0;
    @Getter
    private ItemStack meshStack = ItemStack.field_190927_a;
    @Getter
    private BlockSieve.MeshType meshType = BlockSieve.MeshType.NONE;
    private long lastSieveAction = 0;
    private UUID lastPlayer;

    /**
     * Sets the mesh type in the sieve.
     *
     * @param newMesh The mesh to set
     * @return true if setting is successful.
     */
    public boolean setMesh(ItemStack newMesh) {
        return setMesh(newMesh, false);
    }

    public boolean setMesh(ItemStack newMesh, boolean simulate) {
        if (progress != 0 || currentStack != null)
            return false;

        if (meshStack.func_190926_b()) {
            if (!simulate) {
                meshStack = newMesh.func_77946_l();
                meshType = BlockSieve.MeshType.getMeshTypeByID(newMesh.func_77960_j());

                this.markDirtyClient();
            }
            return true;
        }

        if (!meshStack.func_190926_b() && newMesh.func_190926_b()) {
            //Removing
            if (!simulate) {
                meshStack = ItemStack.field_190927_a;
                meshType = BlockSieve.MeshType.NONE;

                this.markDirtyClient();
            }
            return true;
        }

        return false;

    }

    public boolean addBlock(ItemStack stack) {
        if (currentStack == null && ExNihiloRegistryManager.SIEVE_REGISTRY.canBeSifted(stack)) {
            if (meshStack.func_190926_b())
                return false;
            int meshLevel = meshStack.func_77952_i();
            for (Siftable siftable : ExNihiloRegistryManager.SIEVE_REGISTRY.getDrops(stack)) {
                if (siftable.getMeshLevel() == meshLevel) {
                    currentStack = new BlockInfo(stack);
                    markDirtyClient();
                    return true;
                }
            }
        }

        return false;
    }

    public boolean doSieving(EntityPlayer player, boolean automatedSieving) {
        if (!field_145850_b.field_72995_K) {

            if (currentStack == null) {
                return false;
            }

            // 4 ticks is the same period of holding down right click
            if (func_145831_w().func_82737_E() - lastSieveAction < 4 && !automatedSieving) {
                return false;
            }

            // Really good chance that they're using a macro
            if (!automatedSieving && player != null && func_145831_w().func_82737_E() - lastSieveAction == 0 && lastPlayer.equals(player.func_110124_au())) {
                if (ModConfig.sieve.setFireToMacroUsers) {
                    player.func_70015_d(1);
                }

                player.func_145747_a(new TextComponentString("Bad").func_150255_a(new Style().func_150238_a(TextFormatting.RED).func_150227_a(true)));
            }

            lastSieveAction = func_145831_w().func_82737_E();
            if (player != null) {
                lastPlayer = player.func_110124_au();
            }

            int efficiency = EnchantmentHelper.func_77506_a(ModEnchantments.EFFICIENCY, meshStack);
            efficiency += EnchantmentHelper.func_77506_a(Enchantments.field_185305_q, meshStack);

            int fortune = EnchantmentHelper.func_77506_a(ModEnchantments.FORTUNE, meshStack);
            fortune += EnchantmentHelper.func_77506_a(Enchantments.field_185308_t, meshStack);
            if (player != null) {
                fortune += player.func_184817_da();
            }

            int luckOfTheSea = EnchantmentHelper.func_77506_a(ModEnchantments.LUCK_OF_THE_SEA, meshStack);
            luckOfTheSea += EnchantmentHelper.func_77506_a(Enchantments.field_151370_z, meshStack);

            if (luckOfTheSea > 0 && player != null) {
                luckOfTheSea += player.func_184817_da();
            }

            progress += 10 + 5 * efficiency;
            markDirtyClient();

            if (progress >= 100) {
                List<ItemStack> drops = ExNihiloRegistryManager.SIEVE_REGISTRY.getRewardDrops(rand, currentStack.getBlockState(), meshStack.func_77960_j(), fortune);

                if (drops == null) {
                    drops = new ArrayList<>();
                }

                // Fancy math to make the average fish dropped ~ luckOfTheSea / 2 fish, which is what it was before

                int fishToDrop = (int) Math.round(rand.nextGaussian() + (luckOfTheSea / 2.0));

                fishToDrop = Math.min(fishToDrop, 0);
                fishToDrop = Math.max(fishToDrop, luckOfTheSea);

                for (int i = 0; i < fishToDrop; i++) {
                /*
                 * Gives fish following chances:
                 *  Normal: 43% (3/7)
                 *  Salmon: 29% (2/7)
                 *  Clown:  14% (1/7)
                 *  Puffer: 14% (1/7)
                 */

                    int fishMeta = 0;

                    switch (rand.nextInt(7)) {
                        case 3:
                        case 4:
                            fishMeta = 1;
                            break;
                        case 5:
                            fishMeta = 2;
                            break;
                        case 6:
                            fishMeta = 3;
                            break;
                        default:
                            break;
                    }

                    drops.add(new ItemStack(Items.field_151115_aP, 1, fishMeta));
                }

                drops.forEach(stack -> Util.dropItemInWorld(this, player, stack, 1));

                resetSieve();
                markDirtyClient();
            }
        }
        return true;
    }

    public boolean isSieveSimilar(TileSieve sieve) {
        return sieve != null && !meshStack.func_190926_b() && !sieve.getMeshStack().isEmpty() && meshStack.func_77952_i() == sieve.getMeshStack().getItemDamage() && progress == sieve.getProgress() && BlockInfo.areEqual(currentStack, sieve.getCurrentStack());
    }

    public boolean isSieveSimilarToInput(TileSieve sieve) {
        return !meshStack.func_190926_b() && !sieve.getMeshStack().isEmpty() && meshStack.func_77952_i() == sieve.getMeshStack().getItemDamage() && progress == sieve.getProgress() && sieve.getCurrentStack() == null;
    }

    private void resetSieve() {
        progress = 0;
        currentStack = null;
        markDirtyClient();
    }

    public void validateAutoSieve() {
        if (autoSifter == null || autoSifter.func_145837_r() || !(field_145850_b.func_175625_s(autoSifter.func_174877_v()) instanceof TileAutoSifter || autoSifter.toSift == null)) {
            autoSifter = null;
        }
    }

    @SideOnly(Side.CLIENT)
    public TextureAtlasSprite getTexture() {
        if (currentStack != null) {
            return Util.getTextureFromBlockState(currentStack.getBlockState());
        }
        return null;
    }

    @Override
    public boolean shouldRefresh(World world, BlockPos pos, @Nonnull IBlockState oldState, @Nonnull IBlockState newState) {
        return oldState.func_177230_c() != newState.func_177230_c();
    }

    @Override
    @Nonnull
    public NBTTagCompound func_189515_b(NBTTagCompound tag) {
        if (currentStack != null) {
            NBTTagCompound stackTag = currentStack.writeToNBT(new NBTTagCompound());
            tag.func_74782_a("stack", stackTag);
        }

        if (!meshStack.func_190926_b()) {
            NBTTagCompound meshTag = meshStack.func_77955_b(new NBTTagCompound());
            tag.func_74782_a("mesh", meshTag);
        }

        tag.func_74774_a("progress", progress);

        return super.func_189515_b(tag);
    }

    @Override
    public void func_145839_a(NBTTagCompound tag) {
        if (tag.func_74764_b("stack"))
            currentStack = BlockInfo.readFromNBT(tag.func_74775_l("stack"));
        else
            currentStack = null;

        if (tag.func_74764_b("mesh")) {
            meshStack = new ItemStack(tag.func_74775_l("mesh"));
            meshType = BlockSieve.MeshType.getMeshTypeByID(meshStack.func_77960_j());
        } else {
            meshStack = ItemStack.field_190927_a;
            meshType = BlockSieve.MeshType.NONE;
        }

        progress = tag.func_74771_c("progress");
        super.func_145839_a(tag);
    }

    @Override
    public int hashCode() {
        return Objects.hashCode(field_174879_c.func_177958_n(), field_174879_c.func_177956_o(), field_174879_c.func_177952_p());
    }
}
